"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const os = require("os");
const logs_1 = require("./logs");
const nanoid_1 = require("nanoid");
const jaeger_client_1 = require("jaeger-client");
// HAPI
const hapi_1 = require("@hapi/hapi");
/**
 * Clase para crear un transport, remplazando un broker.
 *
 * @export
 * @class HttpTransport
 * @extends {EventEmitter}
 */
class HttpTransport {
    /**
     * Creates an instance of HttpTransport.
     * @param {string} topic
     * @memberof HttpTransport
     */
    constructor(topic, version, appname, // Nombre de la aplicacion
    serviceName, // Nombre del servicio
    service) {
        /**
         * Puerto en el que inicia el servidor http
         *
         * @private
         * @type {number}
         * @memberof HttpTransport
         */
        this.port = process.env.HTTP_PORT
            ? parseInt(process.env.HTTP_PORT, 0)
            : 3000;
        /**
         * Manejador de logs
         *
         * @private
         * @type {*}
         * @memberof HttpTransport
         */
        this.logs = logs_1.default;
        /**
         * Topico del servicio
         *
         * @private
         * @type {string}
         * @memberof HttpTransport
         */
        this.topicName = '';
        /**
         * Instancia del servicio
         *
         * @private
         * @type {Microservice}
         * @memberof HttpTransport
         */
        this.service = null;
        this.topic = topic;
        this.hostname = os.hostname();
        this.appname = appname;
        this.version = version;
        this.serviceName = serviceName;
        this.service = service; // Instancia del servicio
        // Server
        this.server = new hapi_1.Server({
            port: this.port,
            host: '0.0.0.0',
        });
        // Iniciar el traecing
        this.tracer = jaeger_client_1.initTracer({
            serviceName: `${this.topic}`,
        }, {
            tags: {
                transactionId: this.topic,
            },
        });
    }
    /**
     * Metodo para crear los datos necesarios de la metadata.
     *
     * @private
     * @returns {Metadata}
     * @memberof HttpTransport
     */
    createMetaData(operationID, topicName, topic, transactionID, hostname, asynchronous, id, method, deviceType, filters, span) {
        return {
            uuid: id,
            id_operacion: operationID,
            id_transaction: transactionID,
            intents: 1,
            callback: topic,
            callbacks: [{ method, name: topicName }],
            owner: topic,
            uowner: topic,
            uworker: `${topic}_${operationID}`,
            worker: topic,
            asynchronous,
            mtype: 'input',
            bifurcacion: false,
            time: '',
            from: hostname,
            method,
            deviceType,
            span,
            filters,
        };
    }
    /**
     * Metodo para ejecutar una peticion.
     *
     * @private
     * @param {DataRequest} request
     * @returns {Promise<DataRequest>}
     * @memberof HttpTransport
     */
    process(request) {
        return new Promise((rr, ff) => {
            this.service.message(request, rr); // Ejecutar el servicio y enviar el callback de la promesa
        });
    }
    /**
     * Metodo para preparar los datos requeridos de la peticion
     *
     * @private
     * @param {string} method
     * @param {*} data
     * @param {*} filters
     * @param {string} uuid
     * @param {*} headers
     * @returns {DataRequest}
     * @memberof HttpTransport
     */
    prepare(method, data, filters, uuid, headers, span, operationID) {
        // const message = Buffer.from(JSON.stringify({}),).toString('base64')
        const transactionID = nanoid_1.nanoid();
        // Retornar los datos necesarios.
        return {
            metadata: this.createMetaData(operationID, this.topicName, this.topicName, transactionID, this.hostname, false, uuid, method, 'PC', filters, span),
            data,
            headers,
        };
    }
    /**
     * Handler para las peticiones desde el cliente.
     *
     * @private
     * @param {Request} req
     * @param {ResponseToolkit} resp
     * @memberof HttpTransport
     */
    handler(req, resp) {
        return __awaiter(this, void 0, void 0, function* () {
            // Tiempo inicial de la operacion
            const start = new Date().getTime();
            // Id unico de la operacion
            const operationID = new Date().getTime() / 1000000 + '';
            // Id de la operacion
            const id = req.params.id || null;
            // Metodo
            const method = req.method.toUpperCase();
            // Filters
            const filters = req.query;
            // Data
            const data = req.payload || {};
            // Headers
            const headers = req.headers;
            // Generar el span
            const span = this.tracer.startSpan(`${this.appname}-${this.serviceName}`);
            // Crear la consulta
            const reqOperation = this.prepare(method, data, filters, id, headers, span, operationID);
            try {
                const respOperation = yield this.process(reqOperation);
                const r = respOperation.response;
                span.finish();
                // Validar si ocurrio un error
                if (r.data.response.errorCode || r.data.response.errorCode < 0) {
                    return resp.response(respOperation.response).code(500);
                }
                else {
                    return resp.response(respOperation.response);
                }
            }
            catch (error) {
                this.logs.error(error);
                span.finish();
                return resp
                    .response({
                    meta: {
                        status: 'ERROR',
                        id_transaction: operationID,
                        time_elapsed: new Date().getTime() - start,
                    },
                    data: {
                        response: {
                            errorCode: -9,
                            userMessage: '-35',
                        },
                    },
                })
                    .code(500);
            }
        });
    }
    /**
     *  Metodo que inicia el servidor HTTP.
     *  @public
     */
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            // Escuchar los eventos
            const routes = [
                `/${this.appname}/api/${this.version}/${this.serviceName}`,
                `/${this.appname}/api/${this.version}/${this.serviceName}/:id`,
                `/api/${this.version}/${this.serviceName}`,
                `/api/${this.version}/${this.serviceName}/:id`,
            ];
            routes.forEach((route) => {
                this.server.route({
                    method: '*',
                    path: route,
                    handler: this.handler.bind(this),
                });
            });
            yield this.server.start();
            this.logs.info('Server running on %s', this.server.info.uri);
        });
    }
}
exports.default = HttpTransport;
